@@ -5,7 +5,7 @@ __author__ = "Gina Häußge <osd@foosel.net>"
 __license__ = "GNU Affero General Public License http://www.gnu.org/licenses/agpl.html"
 __copyright__ = "Copyright (C) 2014 The OctoPrint Project - Released under terms of the AGPLv3 License"
 
-
+import copy
 import logging
 
 try:
@@ -25,6 +25,20 @@ from octoprint.util import get_fully_qualified_classname as fqcn
 from octoprint.util import monotonic_time
 from octoprint.util.platform import CLOSE_FDS
 
+EMPTY_RESULT = {
+    "_empty": True,
+    "printingArea": {
+        "minX": 0,
+        "maxX": 0,
+        "minY": 0,
+        "maxY": 0,
+        "minZ": 0,
+        "maxZ": 0,
+    },
+    "dimensions": {"width": 0, "height": 0, "depth": 0},
+    "filament": {},
+}
+
 
 class QueueEntry(
     collections.namedtuple(
@@ -319,6 +333,10 @@ class AbstractAnalysisQueue(object):
                 )
             )
             self._finished_callback(self._current, result)
+        except RuntimeError as exc:
+            self._logger.error(
+                "Analysis for {} ran into error: {}".format(self._current, exc)
+            )
         finally:
             self._current = None
             self._current_progress = None
@@ -415,6 +433,7 @@ class GcodeAnalysisQueue(AbstractAnalysisQueue):
             throttle_lines = settings().getInt(["gcodeAnalysis", "throttle_lines"])
             max_extruders = settings().getInt(["gcodeAnalysis", "maxExtruders"])
             g90_extruder = settings().getBoolean(["feature", "g90InfluencesExtruder"])
+            bed_z = settings().getFloat(["gcodeAnalysis", "bedZ"])
             speedx = self._current.printer_profile["axes"]["x"]["speed"]
             speedy = self._current.printer_profile["axes"]["y"]["speed"]
             offsets = self._current.printer_profile["extruder"]["offsets"]
@@ -430,6 +449,7 @@ class GcodeAnalysisQueue(AbstractAnalysisQueue):
                 "--max-t={}".format(max_extruders),
                 "--throttle={}".format(throttle),
                 "--throttle-lines={}".format(throttle_lines),
+                "--bed-z={}".format(bed_z),
             ]
             for offset in offsets[1:]:
                 command += ["--offset", str(offset[0]), str(offset[1])]
@@ -477,24 +497,30 @@ class GcodeAnalysisQueue(AbstractAnalysisQueue):
             output = p.stdout.text
             self._logger.debug("Got output: {!r}".format(output))
 
-            if "RESULTS:" not in output:
-                raise RuntimeError("No analysis result found")
-
-            _, output = output.split("RESULTS:")
-            analysis = yaml.safe_load(output)
-
             result = {}
-            result["printingArea"] = analysis["printing_area"]
-            result["dimensions"] = analysis["dimensions"]
-            if analysis["total_time"]:
-                result["estimatedPrintTime"] = analysis["total_time"] * 60
-            if analysis["extrusion_length"]:
-                result["filament"] = {}
-                for i in range(len(analysis["extrusion_length"])):
-                    result["filament"]["tool%d" % i] = {
-                        "length": analysis["extrusion_length"][i],
-                        "volume": analysis["extrusion_volume"][i],
-                    }
+            if "ERROR:" in output:
+                _, error = output.split("ERROR:")
+                raise RuntimeError(error.strip())
+            elif "EMPTY:" in output:
+                self._logger.info("Result is empty, no extrusions found")
+                result = copy.deepcopy(EMPTY_RESULT)
+            elif "RESULTS:" not in output:
+                raise RuntimeError("No analysis result found")
+            else:
+                _, output = output.split("RESULTS:")
+                analysis = yaml.safe_load(output)
+
+                result["printingArea"] = analysis["printing_area"]
+                result["dimensions"] = analysis["dimensions"]
+                if analysis["total_time"]:
+                    result["estimatedPrintTime"] = analysis["total_time"] * 60
+                if analysis["extrusion_length"]:
+                    result["filament"] = {}
+                    for i in range(len(analysis["extrusion_length"])):
+                        result["filament"]["tool%d" % i] = {
+                            "length": analysis["extrusion_length"][i],
+                            "volume": analysis["extrusion_volume"][i],
+                        }
 
             if self._current.analysis and isinstance(self._current.analysis, dict):
                 return dict_merge(result, self._current.analysis)
