@@ -11,7 +11,7 @@ import os
 import threading
 
 import psutil
-from flask import jsonify, make_response, request, url_for
+from flask import abort, jsonify, make_response, request, url_for
 
 import octoprint.filemanager
 import octoprint.filemanager.storage
@@ -37,6 +37,11 @@ from octoprint.server.util.flask import (
 from octoprint.settings import settings, valid_boolean_trues
 from octoprint.util import sv, time_this
 
+try:
+    from urllib.parse import quote as urlquote
+except ImportError:
+    from urllib import quote as urlquote  # noqa: F401
+
 # ~~ GCODE file handling
 
 _file_cache = {}
@@ -157,12 +162,60 @@ def readGcodeFiles():
         recursive=recursive,
         allow_from_cache=not force,
     )
-    files.extend(_getFileList(FileDestinations.SDCARD))
+    files.extend(_getFileList(FileDestinations.SDCARD, allow_from_cache=not force))
 
     usage = psutil.disk_usage(settings().getBaseFolder("uploads", check_writable=False))
     return jsonify(files=files, free=usage.free, total=usage.total)
 
 
+@api.route("/files/test", methods=["POST"])
+@Permissions.FILES_LIST.require(403)
+def runFilesTest():
+    valid_commands = {
+        "sanitize": ["storage", "path", "filename"],
+        "exists": ["storage", "path", "filename"],
+    }
+
+    command, data, response = get_json_command_from_request(request, valid_commands)
+    if response is not None:
+        return response
+
+    def sanitize(storage, path, filename):
+        sanitized_path = fileManager.sanitize_path(storage, path)
+        sanitized_name = fileManager.sanitize_name(storage, filename)
+        joined = fileManager.join_path(storage, sanitized_path, sanitized_name)
+        return sanitized_path, sanitized_name, joined
+
+    if command == "sanitize":
+        _, _, sanitized = sanitize(data["storage"], data["path"], data["filename"])
+        return jsonify(sanitized=sanitized)
+    elif command == "exists":
+        storage = data["storage"]
+        path = data["path"]
+        filename = data["filename"]
+
+        sanitized_path, _, sanitized = sanitize(storage, path, filename)
+
+        exists = fileManager.file_exists(storage, sanitized)
+        if exists:
+            suggestion = filename
+            name, ext = os.path.splitext(filename)
+            counter = 0
+            while fileManager.file_exists(
+                storage,
+                fileManager.join_path(
+                    storage,
+                    sanitized_path,
+                    fileManager.sanitize_name(storage, suggestion),
+                ),
+            ):
+                counter += 1
+                suggestion = name + "_{}".format(counter) + ext
+            return jsonify(exists=True, suggestion=suggestion)
+        else:
+            return jsonify(exists=False)
+
+
 @api.route("/files/<string:origin>", methods=["GET"])
 @Permissions.FILES_LIST.require(403)
 @with_revalidation_checking(
@@ -180,7 +233,7 @@ def readGcodeFiles():
 )
 def readGcodeFilesForOrigin(origin):
     if origin not in [FileDestinations.LOCAL, FileDestinations.SDCARD]:
-        return make_response("Unknown origin: %s" % origin, 404)
+        abort(404)
 
     filter = request.values.get("filter", False)
     recursive = request.values.get("recursive", "false") in valid_boolean_trues
@@ -216,7 +269,10 @@ def readGcodeFilesForOrigin(origin):
 )
 def readGcodeFile(target, filename):
     if target not in [FileDestinations.LOCAL, FileDestinations.SDCARD]:
-        return make_response("Unknown target: %s" % target, 404)
+        abort(404)
+
+    if not _validate(target, filename):
+        abort(404)
 
     recursive = False
     if "recursive" in request.values:
@@ -224,7 +280,7 @@ def readGcodeFile(target, filename):
 
     file = _getFileDetails(target, filename, recursive=recursive)
     if not file:
-        return make_response("File not found on '%s': %s" % (target, filename), 404)
+        abort(404)
 
     return jsonify(file)
 
@@ -244,12 +300,14 @@ def _getFileDetails(origin, path, recursive=True):
     logtarget=__name__ + ".timings",
     message="{func}({func_args},{func_kwargs}) took {timing:.2f}ms",
     incl_func_args=True,
+    log_enter=True,
+    message_enter="Entering {func}({func_args},{func_kwargs})...",
 )
 def _getFileList(
     origin, path=None, filter=None, recursive=False, level=0, allow_from_cache=True
 ):
     if origin == FileDestinations.SDCARD:
-        sdFileList = printer.get_sd_files()
+        sdFileList = printer.get_sd_files(refresh=not allow_from_cache)
 
         files = []
         if sdFileList is not None:
@@ -272,7 +330,7 @@ def _getFileList(
                         "resource": url_for(
                             ".readGcodeFile",
                             target=FileDestinations.SDCARD,
-                            filename=f["name"],
+                            filename=urlquote(f["name"]),
                             _external=True,
                         )
                     },
@@ -334,7 +392,7 @@ def _getFileList(
                         "resource": url_for(
                             ".readGcodeFile",
                             target=FileDestinations.LOCAL,
-                            filename=path + file_or_folder["name"],
+                            filename=urlquote(path + file_or_folder["name"]),
                             _external=True,
                         )
                     }
@@ -388,14 +446,14 @@ def _getFileList(
                         "resource": url_for(
                             ".readGcodeFile",
                             target=FileDestinations.LOCAL,
-                            filename=file_or_folder["path"],
+                            filename=urlquote(file_or_folder["path"]),
                             _external=True,
                         ),
                         "download": url_for("index", _external=True)
                         + "downloads/files/"
                         + FileDestinations.LOCAL
                         + "/"
-                        + file_or_folder["path"],
+                        + urlquote(file_or_folder["path"]),
                     }
 
                 result.append(file_or_folder)
@@ -450,7 +508,7 @@ def uploadGcodeFile(target):
     )
     if input_upload_name in request.values and input_upload_path in request.values:
         if target not in [FileDestinations.LOCAL, FileDestinations.SDCARD]:
-            return make_response("Unknown target: %s" % target, 404)
+            abort(404)
 
         upload = octoprint.filemanager.util.DiskFileWrapper(
             request.values[input_upload_name], request.values[input_upload_path]
@@ -464,12 +522,12 @@ def uploadGcodeFile(target):
             try:
                 userdata = json.loads(request.values["userdata"])
             except Exception:
-                return make_response("userdata contains invalid JSON", 400)
+                abort(400, description="userdata contains invalid JSON")
 
         if target == FileDestinations.SDCARD and not settings().getBoolean(
             ["feature", "sdSupport"]
         ):
-            return make_response("SD card support is disabled", 404)
+            abort(404)
 
         sd = target == FileDestinations.SDCARD
         selectAfterUpload = (
@@ -489,14 +547,12 @@ def uploadGcodeFile(target):
                 printer.is_operational()
                 and not (printer.is_printing() or printer.is_paused())
             ):
-                return make_response(
-                    "Can not upload to SD card, printer is either not operational or already busy",
+                abort(
                     409,
+                    description="Can not upload to SD card, printer is either not operational or already busy",
                 )
             if not printer.is_sd_ready():
-                return make_response(
-                    "Can not upload to SD card, not yet initialized", 409
-                )
+                abort(409, description="Can not upload to SD card, not yet initialized")
 
         # determine future filename of file to be uploaded, abort if it can't be uploaded
         try:
@@ -504,6 +560,11 @@ def uploadGcodeFile(target):
             canonPath, canonFilename = fileManager.canonicalize(
                 FileDestinations.LOCAL, upload.filename
             )
+            if request.values.get("path"):
+                canonPath = request.values.get("path")
+            if request.values.get("filename"):
+                canonFilename = request.values.get("filename")
+
             futurePath = fileManager.sanitize_path(FileDestinations.LOCAL, canonPath)
             futureFilename = fileManager.sanitize_name(
                 FileDestinations.LOCAL, canonFilename
@@ -514,15 +575,7 @@ def uploadGcodeFile(target):
             futureFilename = None
 
         if futureFilename is None:
-            return make_response(
-                "Can not upload file %s, wrong format?" % upload.filename, 415
-            )
-
-        if "path" in request.values and request.values["path"]:
-            # we currently only support uploads to sdcard via local, so first target is local instead of "target"
-            futurePath = fileManager.sanitize_path(
-                FileDestinations.LOCAL, request.values["path"]
-            )
+            abort(415, description="Can not upload file, wrong format?")
 
         # prohibit overwriting currently selected file while it's being printed
         futureFullPath = fileManager.join_path(
@@ -533,12 +586,17 @@ def uploadGcodeFile(target):
         )
 
         if not printer.can_modify_file(futureFullPathInStorage, sd):
-            return make_response(
-                "Trying to overwrite file that is currently being printed: %s"
-                % futureFullPath,
+            abort(
                 409,
+                description="Trying to overwrite file that is currently being printed",
             )
 
+        if (
+            fileManager.file_exists(FileDestinations.LOCAL, futureFullPathInStorage)
+            and request.values.get("noOverwrite") in valid_boolean_trues
+        ):
+            abort(409, description="File already exists and noOverwrite was set")
+
         reselect = printer.is_current_file(futureFullPathInStorage, sd)
 
         user = current_user.get_name()
@@ -594,16 +652,9 @@ def uploadGcodeFile(target):
             )
         except octoprint.filemanager.storage.StorageError as e:
             if e.code == octoprint.filemanager.storage.StorageError.INVALID_FILE:
-                return make_response(
-                    'Could not upload the file "{}", invalid type'.format(
-                        upload.filename
-                    ),
-                    400,
-                )
+                abort(400, description="Could not upload file, invalid type")
             else:
-                return make_response(
-                    'Could not upload the file "{}"'.format(upload.filename), 500
-                )
+                abort(500, description="Could not upload file")
 
         if octoprint.filemanager.valid_file_type(added_file, "stl"):
             filename = added_file
@@ -641,7 +692,7 @@ def uploadGcodeFile(target):
         location = url_for(
             ".readGcodeFile",
             target=FileDestinations.LOCAL,
-            filename=filename,
+            filename=urlquote(filename),
             _external=True,
         )
         files.update(
@@ -656,7 +707,7 @@ def uploadGcodeFile(target):
                         + "downloads/files/"
                         + FileDestinations.LOCAL
                         + "/"
-                        + filename,
+                        + urlquote(filename),
                     },
                 }
             }
@@ -666,7 +717,7 @@ def uploadGcodeFile(target):
             location = url_for(
                 ".readGcodeFile",
                 target=FileDestinations.SDCARD,
-                filename=sdFilename,
+                filename=urlquote(sdFilename),
                 _external=True,
             )
             files.update(
@@ -688,13 +739,13 @@ def uploadGcodeFile(target):
         foldername = request.values["foldername"]
 
         if target not in [FileDestinations.LOCAL]:
-            return make_response("Unknown target: %s" % target, 400)
+            abort(400, description="target is invalid")
 
         canonPath, canonName = fileManager.canonicalize(target, foldername)
         futurePath = fileManager.sanitize_path(target, canonPath)
         futureName = fileManager.sanitize_name(target, canonName)
         if not futureName or not futurePath:
-            return make_response("Can't create a folder with an empty name", 400)
+            abort(400, description="folder name is empty")
 
         if "path" in request.values and request.values["path"]:
             futurePath = fileManager.sanitize_path(
@@ -703,10 +754,7 @@ def uploadGcodeFile(target):
 
         futureFullPath = fileManager.join_path(target, futurePath, futureName)
         if octoprint.filemanager.valid_file_type(futureName):
-            return make_response(
-                "Can't create a folder named %s, please try another name" % futureName,
-                409,
-            )
+            abort(409, description="Can't create folder, please try another name")
 
         try:
             added_folder = fileManager.add_folder(
@@ -714,16 +762,14 @@ def uploadGcodeFile(target):
             )
         except octoprint.filemanager.storage.StorageError as e:
             if e.code == octoprint.filemanager.storage.StorageError.INVALID_DIRECTORY:
-                return make_response(
-                    "Could not create folder {}, invalid directory".format(futureName)
-                )
+                abort(400, description="Could not create folder, invalid directory")
             else:
-                return make_response("Could not create folder {}".format(futureName))
+                abort(500, description="Could not create folder")
 
         location = url_for(
             ".readGcodeFile",
             target=FileDestinations.LOCAL,
-            filename=added_folder,
+            filename=urlquote(added_folder),
             _external=True,
         )
         folder = {
@@ -738,18 +784,22 @@ def uploadGcodeFile(target):
         return r
 
     else:
-        return make_response("No file to upload and no folder to create", 400)
+        abort(400, description="No file to upload and no folder to create")
 
 
 @api.route("/files/<string:target>/<path:filename>", methods=["POST"])
 @no_firstrun_access
 def gcodeFileCommand(filename, target):
     if target not in [FileDestinations.LOCAL, FileDestinations.SDCARD]:
-        return make_response("Unknown target: %s" % target, 404)
+        abort(404)
+
+    if not _validate(target, filename):
+        abort(404)
 
     # valid file commands, dict mapping command name to mandatory parameters
     valid_commands = {
         "select": [],
+        "unselect": [],
         "slice": [],
         "analyse": [],
         "copy": ["destination"],
@@ -765,31 +815,28 @@ def gcodeFileCommand(filename, target):
     if command == "select":
         with Permissions.FILES_SELECT.require(403):
             if not _verifyFileExists(target, filename):
-                return make_response(
-                    "File not found on '%s': %s" % (target, filename), 404
-                )
+                abort(404)
 
             # selects/loads a file
             if not octoprint.filemanager.valid_file_type(filename, type="machinecode"):
-                return make_response(
-                    "Cannot select {filename} for printing, not a machinecode file".format(
-                        **locals()
-                    ),
+                abort(
                     415,
+                    description="Cannot select file for printing, not a machinecode file",
                 )
 
             if not printer.is_ready():
-                return make_response(
-                    "Printer is already printing, cannot select a new file", 409
+                abort(
+                    409,
+                    description="Printer is already printing, cannot select a new file",
                 )
 
             printAfterLoading = False
             if "print" in data and data["print"] in valid_boolean_trues:
                 with Permissions.PRINT.require(403):
                     if not printer.is_operational():
-                        return make_response(
-                            "Printer is not operational, cannot directly start printing",
+                        abort(
                             409,
+                            description="Printer is not operational, cannot directly start printing",
                         )
                     printAfterLoading = True
 
@@ -801,12 +848,30 @@ def gcodeFileCommand(filename, target):
                 filenameToSelect = fileManager.path_on_disk(target, filename)
             printer.select_file(filenameToSelect, sd, printAfterLoading, user)
 
+    elif command == "unselect":
+        with Permissions.FILES_SELECT.require(403):
+            if not printer.is_ready():
+                return make_response(
+                    "Printer is already printing, cannot unselect current file", 409
+                )
+
+            _, currentFilename = _getCurrentFile()
+            if currentFilename is None:
+                return make_response(
+                    "Cannot unselect current file when there is no file selected", 409
+                )
+
+            if filename != currentFilename and filename != "current":
+                return make_response(
+                    "Only the currently selected file can be unselected", 400
+                )
+
+            printer.unselect_file()
+
     elif command == "slice":
         with Permissions.SLICE.require(403):
             if not _verifyFileExists(target, filename):
-                return make_response(
-                    "File not found on '%s': %s" % (target, filename), 404
-                )
+                abort(404)
 
             try:
                 if "slicer" in data:
@@ -819,14 +884,9 @@ def gcodeFileCommand(filename, target):
                     slicer_instance = slicingManager.get_slicer("cura")
 
                 else:
-                    return make_response(
-                        "Cannot slice {filename}, no slicer available".format(**locals()),
-                        415,
-                    )
-            except octoprint.slicing.UnknownSlicer as e:
-                return make_response(
-                    "Slicer {slicer} is not available".format(slicer=e.slicer), 400
-                )
+                    abort(415, description="Cannot slice file, no slicer available")
+            except octoprint.slicing.UnknownSlicer:
+                abort(404)
 
             if not any(
                 [
@@ -836,9 +896,7 @@ def gcodeFileCommand(filename, target):
                     )
                 ]
             ):
-                return make_response(
-                    "Cannot slice {filename}, not a model file".format(**locals()), 415
-                )
+                abort(415, description="Cannot slice file, not a model file")
 
             cores = psutil.cpu_count()
             if (
@@ -847,11 +905,9 @@ def gcodeFileCommand(filename, target):
                 and (cores is None or cores < 2)
             ):
                 # slicer runs on same device as OctoPrint, slicing while printing is hence disabled
-                return make_response(
-                    "Cannot slice on {slicer} while printing on single core systems or systems of unknown core count due to performance reasons".format(
-                        **locals()
-                    ),
+                abort(
                     409,
+                    description="Cannot slice on this slicer while printing on single core systems or systems of unknown core count due to performance reasons",
                 )
 
             if "destination" in data and data["destination"]:
@@ -895,10 +951,9 @@ def gcodeFileCommand(filename, target):
                 and currentOrigin == target
                 and (printer.is_printing() or printer.is_paused())
             ):
-                make_response(
-                    "Trying to slice into file that is currently being printed: %s"
-                    % full_path,
+                abort(
                     409,
+                    description="Trying to slice into file that is currently being printed",
                 )
 
             if "profile" in data and data["profile"]:
@@ -928,17 +983,18 @@ def gcodeFileCommand(filename, target):
             select_after_slicing = False
             if "select" in data and data["select"] in valid_boolean_trues:
                 if not printer.is_operational():
-                    return make_response(
-                        "Printer is not operational, cannot directly select for printing",
+                    abort(
                         409,
+                        description="Printer is not operational, cannot directly select for printing",
                     )
                 select_after_slicing = True
 
             print_after_slicing = False
             if "print" in data and data["print"] in valid_boolean_trues:
                 if not printer.is_operational():
-                    return make_response(
-                        "Printer is not operational, cannot directly start printing", 409
+                    abort(
+                        409,
+                        description="Printer is not operational, cannot directly start printing",
                     )
                 select_after_slicing = print_after_slicing = True
 
@@ -980,13 +1036,13 @@ def gcodeFileCommand(filename, target):
                     ),
                 )
             except octoprint.slicing.UnknownProfile:
-                return make_response(
-                    "Profile {profile} doesn't exist".format(**locals()), 400
-                )
+                abort(404, description="Unknown profile")
 
-            files = {}
             location = url_for(
-                ".readGcodeFile", target=target, filename=full_path, _external=True
+                ".readGcodeFile",
+                target=target,
+                filename=urlquote(full_path),
+                _external=True,
             )
             result = {
                 "name": destination,
@@ -999,7 +1055,7 @@ def gcodeFileCommand(filename, target):
                     + "downloads/files/"
                     + target
                     + "/"
-                    + full_path,
+                    + urlquote(full_path),
                 },
             }
 
@@ -1010,9 +1066,7 @@ def gcodeFileCommand(filename, target):
     elif command == "analyse":
         with Permissions.FILES_UPLOAD.require(403):
             if not _verifyFileExists(target, filename):
-                return make_response(
-                    "File not found on '%s': %s" % (target, filename), 404
-                )
+                abort(404)
 
             printer_profile = None
             if "printerProfile" in data and data["printerProfile"]:
@@ -1021,24 +1075,18 @@ def gcodeFileCommand(filename, target):
             if not fileManager.analyse(
                 target, filename, printer_profile_id=printer_profile
             ):
-                return make_response(
-                    "No analysis possible for {} on {}".format(filename, target), 400
-                )
+                abort(400, description="No analysis possible")
 
     elif command == "copy" or command == "move":
         with Permissions.FILES_UPLOAD.require(403):
             # Copy and move are only possible on local storage
             if target not in [FileDestinations.LOCAL]:
-                return make_response(
-                    "Unsupported target for {}: {}".format(command, target), 400
-                )
+                abort(400, description="Unsupported target for {}".format(command))
 
             if not _verifyFileExists(target, filename) and not _verifyFolderExists(
                 target, filename
             ):
-                return make_response(
-                    "File or folder not found on {}: {}".format(target, filename), 404
-                )
+                abort(404)
 
             path, name = fileManager.split_path(target, filename)
 
@@ -1048,33 +1096,32 @@ def gcodeFileCommand(filename, target):
                 target, dst_path, fileManager.sanitize_name(target, dst_name)
             )
 
-            if (
-                _verifyFolderExists(target, destination)
-                and sanitized_destination != filename
-            ):
-                # destination is an existing folder and not ourselves (= display rename), we'll assume we are supposed
-                # to move filename to this folder under the same name
-                destination = fileManager.join_path(target, destination, name)
+            # Check for exception thrown by _verifyFolderExists, if outside the root directory
+            try:
+                if (
+                    _verifyFolderExists(target, destination)
+                    and sanitized_destination != filename
+                ):
+                    # destination is an existing folder and not ourselves (= display rename), we'll assume we are supposed
+                    # to move filename to this folder under the same name
+                    destination = fileManager.join_path(target, destination, name)
 
-            if _verifyFileExists(target, destination) or _verifyFolderExists(
-                target, destination
-            ):
-                return make_response(
-                    "File or folder does already exist on {}: {}".format(
-                        target, destination
-                    ),
-                    409,
+                if _verifyFileExists(target, destination) or _verifyFolderExists(
+                    target, destination
+                ):
+                    abort(409, description="File or folder does already exist")
+
+            except Exception:
+                abort(
+                    409, description="Exception thrown by storage, bad folder/file name?"
                 )
 
             is_file = fileManager.file_exists(target, filename)
             is_folder = fileManager.folder_exists(target, filename)
 
             if not (is_file or is_folder):
-                return make_response(
-                    "{} on {} is neither file or folder, can't {}".format(
-                        filename, target, command
-                    ),
-                    400,
+                abort(
+                    400, description="Neither file nor folder, can't {}".format(command)
                 )
 
             if command == "copy":
@@ -1082,12 +1129,7 @@ def gcodeFileCommand(filename, target):
                 if _verifyFileExists(target, destination) or _verifyFolderExists(
                     target, destination
                 ):
-                    return make_response(
-                        "File or folder does already exist on {}: {}".format(
-                            target, destination
-                        ),
-                        409,
-                    )
+                    abort(409, description="File or folder does already exist")
 
                 if is_file:
                     fileManager.copy_file(target, filename, destination)
@@ -1097,11 +1139,9 @@ def gcodeFileCommand(filename, target):
             elif command == "move":
                 with Permissions.FILES_DELETE.require(403):
                     if _isBusy(target, filename):
-                        return make_response(
-                            "Trying to move a file or folder that is currently in use: {}".format(
-                                filename
-                            ),
+                        abort(
                             409,
+                            description="Trying to move a file or folder that is currently in use",
                         )
 
                     # destination already there AND not ourselves (= display rename)? error...
@@ -1109,12 +1149,7 @@ def gcodeFileCommand(filename, target):
                         _verifyFileExists(target, destination)
                         or _verifyFolderExists(target, destination)
                     ) and sanitized_destination != filename:
-                        return make_response(
-                            "File or folder does already exist on {}: {}".format(
-                                target, destination
-                            ),
-                            409,
-                        )
+                        abort(409, description="File or folder does already exist")
 
                     # deselect the file if it's currently selected
                     currentOrigin, currentFilename = _getCurrentFile()
@@ -1127,7 +1162,10 @@ def gcodeFileCommand(filename, target):
                         fileManager.move_folder(target, filename, destination)
 
             location = url_for(
-                ".readGcodeFile", target=target, filename=destination, _external=True
+                ".readGcodeFile",
+                target=target,
+                filename=urlquote(destination),
+                _external=True,
             )
             result = {
                 "name": name,
@@ -1141,7 +1179,7 @@ def gcodeFileCommand(filename, target):
                     + "downloads/files/"
                     + target
                     + "/"
-                    + destination
+                    + urlquote(destination)
                 )
 
             r = make_response(jsonify(result), 201)
@@ -1155,21 +1193,20 @@ def gcodeFileCommand(filename, target):
 @no_firstrun_access
 @Permissions.FILES_DELETE.require(403)
 def deleteGcodeFile(filename, target):
+    if not _validate(target, filename):
+        abort(404)
+
     if not _verifyFileExists(target, filename) and not _verifyFolderExists(
         target, filename
     ):
-        return make_response(
-            "File/Folder not found on '%s': %s" % (target, filename), 404
-        )
+        abort(404)
+
+    if target not in [FileDestinations.LOCAL, FileDestinations.SDCARD]:
+        abort(404)
 
     if _verifyFileExists(target, filename):
-        if target not in [FileDestinations.LOCAL, FileDestinations.SDCARD]:
-            return make_response("Unknown target: %s" % target, 400)
-
         if _isBusy(target, filename):
-            return make_response(
-                "Trying to delete a file that is currently in use: %s" % filename, 409
-            )
+            abort(409, description="Trying to delete a file that is currently in use")
 
         # deselect the file if it's currently selected
         currentOrigin, currentPath = _getCurrentFile()
@@ -1187,14 +1224,10 @@ def deleteGcodeFile(filename, target):
             fileManager.remove_file(target, filename)
 
     elif _verifyFolderExists(target, filename):
-        if target not in [FileDestinations.LOCAL]:
-            return make_response("Unknown target: %s" % target, 400)
-
         if _isBusy(target, filename):
-            return make_response(
-                "Trying to delete a folder that contains a file that is currently in use: %s"
-                % filename,
+            abort(
                 409,
+                description="Trying to delete a folder that contains a file that is currently in use",
             )
 
         # deselect the file if it's currently selected
@@ -1225,6 +1258,16 @@ def _getCurrentFile():
         return None, None
 
 
+def _validate(target, filename):
+    if target == FileDestinations.SDCARD:
+        # we make no assumptions about the shape of valid SDCard file names
+        return True
+    else:
+        return filename == "/".join(
+            map(lambda x: fileManager.sanitize_name(target, x), filename.split("/"))
+        )
+
+
 class WerkzeugFileWrapper(octoprint.filemanager.util.AbstractFileWrapper):
     """
     A wrapper around a Werkzeug ``FileStorage`` object.
