@@ -1,6 +1,4 @@
 # -*- coding: utf-8 -*-
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 """
 This module represents OctoPrint's settings management. Within this module the default settings for the core
 application are defined and the instance of the :class:`Settings` is held, which offers getter and setter
@@ -18,12 +16,14 @@ of various types and the configuration file itself.
    :members:
    :undoc-members:
 """
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 __author__ = "Gina Häußge <osd@foosel.net>"
 __license__ = "GNU Affero General Public License http://www.gnu.org/licenses/agpl.html"
 __copyright__ = "Copyright (C) 2014 The OctoPrint Project - Released under terms of the AGPLv3 License"
 
 import copy
+import fnmatch
 import io
 import logging
 import os
@@ -47,6 +47,11 @@ try:
 except ImportError:
     from collections import KeysView
 
+try:
+    from os import scandir
+except ImportError:
+    from scandir import scandir
+
 from octoprint.util import (
     CaseInsensitiveSet,
     atomic_write,
@@ -60,7 +65,7 @@ _APPNAME = "OctoPrint"
 _instance = None
 
 
-def settings(init=False, basedir=None, configfile=None):
+def settings(init=False, basedir=None, configfile=None, overlays=None):
     """
     Factory method for initially constructing and consecutively retrieving the :class:`~octoprint.settings.Settings`
     singleton.
@@ -75,6 +80,7 @@ def settings(init=False, basedir=None, configfile=None):
             ``~/Library/Application Support/OctoPrint`` on MacOS.
         configfile (str): Path of the configuration file (``config.yaml``) to work on. If not set the default will
             be used: ``<basedir>/config.yaml`` for ``basedir`` as defined above.
+        overlays (list): List of paths to config overlays to put between default settings and config.yaml
 
     Returns:
         Settings: The fully initialized :class:`Settings` instance.
@@ -89,7 +95,9 @@ def settings(init=False, basedir=None, configfile=None):
 
     else:
         if init:
-            _instance = Settings(configfile=configfile, basedir=basedir)
+            _instance = Settings(
+                configfile=configfile, basedir=basedir, overlays=overlays
+            )
         else:
             raise ValueError("Settings not initialized yet")
 
@@ -101,6 +109,7 @@ default_settings = {
         "port": None,
         "baudrate": None,
         "exclusive": True,
+        "lowLatency": False,
         "autoconnect": False,
         "log": False,
         "timeout": {
@@ -114,6 +123,7 @@ default_settings = {
             "temperatureAutoreport": 2,
             "sdStatus": 1,
             "sdStatusAutoreport": 1,
+            "posAutoreport": 5,
             "resendOk": 0.5,
             "baudrateDetectionPause": 1.0,
             "positionLogWait": 10.0,
@@ -126,6 +136,7 @@ default_settings = {
         "blacklistedBaudrates": [],
         "longRunningCommands": ["G4", "G28", "G29", "G30", "G32", "M400", "M226", "M600"],
         "blockedCommands": ["M0", "M1"],
+        "ignoredCommands": [],
         "pausingCommands": ["M0", "M1", "M25"],
         "emergencyCommands": ["M112", "M108", "M410"],
         "checksumRequiringCommands": ["M110"],
@@ -133,6 +144,7 @@ default_settings = {
         "disconnectOnErrors": True,
         "ignoreErrorsFromFirmware": False,
         "terminalLogSize": 20,
+        "lastLineBufferSize": 50,
         "logResends": True,
         "supportResendsWithoutOk": "detect",
         "logPositionOnPause": True,
@@ -145,6 +157,7 @@ default_settings = {
         "unknownCommandsNeedAck": False,
         "sdRelativePath": False,
         "sdAlwaysAvailable": False,
+        "sdLowerCase": False,
         "maxNotSdPrinting": 2,
         "swallowOkAfterResend": True,
         "repetierTargetTemp": False,
@@ -165,8 +178,10 @@ default_settings = {
         "capabilities": {
             "autoreport_temp": True,
             "autoreport_sdstatus": True,
+            "autoreport_pos": True,
             "busy_protocol": True,
             "emergency_parser": True,
+            "extended_m20": True,
         },
         "resendRatioThreshold": 10,
         "resendRatioStart": 100,
@@ -211,7 +226,7 @@ default_settings = {
         "onlineCheck": {
             "enabled": None,
             "interval": 15 * 60,  # 15 min
-            "host": "8.8.8.8",
+            "host": "1.1.1.1",
             "port": 53,
             "name": "octoprint.org",
         },
@@ -246,6 +261,7 @@ default_settings = {
         "flipH": False,
         "flipV": False,
         "rotate90": False,
+        "ffmpegCommandline": '{ffmpeg} -r {fps} -i "{input}" -vcodec {videocodec} -threads {threads} -b:v {bitrate} -f {containerformat} -y {filters} "{output}"',
         "timelapse": {
             "type": "off",
             "options": {},
@@ -253,6 +269,7 @@ default_settings = {
             "fps": 25,
         },
         "cleanTmpAfterDays": 7,
+        "cacheBuster": False,
     },
     "gcodeAnalysis": {
         "maxExtruders": 10,
@@ -260,6 +277,7 @@ default_settings = {
         "throttle_highprio": 0.0,
         "throttle_lines": 100,
         "runAt": "idle",  # 'never', 'idle', 'always'
+        "bedZ": 0.0,
     },
     "feature": {
         "temperatureGraph": True,
@@ -269,8 +287,10 @@ default_settings = {
         "modelSizeDetection": True,
         "printStartConfirmation": False,
         "printCancelConfirmation": True,
+        "uploadOverwriteConfirmation": True,
         "autoUppercaseBlacklist": ["M117", "M118"],
         "g90InfluencesExtruder": False,
+        "enforceReallyUniversalFilenames": False,
     },
     "folder": {
         "uploads": None,
@@ -319,7 +339,13 @@ default_settings = {
                     "plugin_pi_support",
                     "login",
                 ],
-                "sidebar": ["plugin_firmware_check", "connection", "state", "files"],
+                "sidebar": [
+                    "plugin_firmware_check_warning",
+                    "plugin_firmware_check_info",
+                    "connection",
+                    "state",
+                    "files",
+                ],
                 "tab": [
                     "temperature",
                     "control",
@@ -349,13 +375,19 @@ default_settings = {
                     "plugin_pluginmanager",
                     "plugin_softwareupdate",
                     "plugin_announcements",
+                    "plugin_eventmanager",
                     "plugin_backup",
                     "plugin_tracking",
                     "plugin_errortracking",
                     "plugin_pi_support",
                 ],
                 "usersettings": ["access", "interface"],
-                "wizard": ["plugin_backup", "plugin_corewizard_acl"],
+                "wizard": [
+                    "plugin_softwareupdate_update",
+                    "plugin_backup",
+                    "plugin_corewizard_acl",
+                    "plugin_corewizard_onlinecheck",
+                ],
                 "about": [
                     "about",
                     "plugin_pi_support",
@@ -402,12 +434,16 @@ default_settings = {
     "terminalFilters": [
         {
             "name": "Suppress temperature messages",
-            "regex": r"(Send: (N\d+\s+)?M105)|(Recv:\s+(ok\s+((P|B|N)\d+\s+)*)?(B|T\d*):\d+)",
+            "regex": r"(Send: (N\d+\s+)?M105)|(Recv:\s+(ok\s+([PBN]\d+\s+)*)?([BCLPR]|T\d*):-?\d+)",
         },
         {
             "name": "Suppress SD status messages",
             "regex": r"(Send: (N\d+\s+)?M27)|(Recv: SD printing byte)|(Recv: Not SD printing)",
         },
+        {
+            "name": "Suppress position messages",
+            "regex": r"(Send:\s+(N\d+\s+)?M114)|(Recv:\s+(ok\s+)?X:[+-]?([0-9]*[.])?[0-9]+\s+Y:[+-]?([0-9]*[.])?[0-9]+\s+Z:[+-]?([0-9]*[.])?[0-9]+\s+E\d*:[+-]?([0-9]*[.])?[0-9]+).*",
+        },
         {"name": "Suppress wait responses", "regex": r"Recv: wait"},
         {
             "name": "Suppress processing responses",
@@ -658,14 +694,18 @@ class Settings(object):
 
     OVERLAY_KEY = "__overlay__"
 
-    def __init__(self, configfile=None, basedir=None):
+    def __init__(self, configfile=None, basedir=None, overlays=None):
         self._logger = logging.getLogger(__name__)
 
         self._basedir = None
 
+        if overlays is None:
+            overlays = []
+
         assert isinstance(default_settings, dict)
 
         self._map = HierarchicalChainMap({}, default_settings)
+        self.load_overlays(overlays)
 
         self._config = None
         self._dirty = False
@@ -717,11 +757,17 @@ class Settings(object):
     def sanity_check_folders(self, folders=None):
         if folders is None:
             folders = default_settings["folder"].keys()
+
+        folder_map = {}
         for folder in folders:
-            self.getBaseFolder(
-                folder, check_writable=True, deep_check_writable=True, log_error=True
+            folder_map[folder] = self.getBaseFolder(
+                folder, check_writable=True, deep_check_writable=True
             )
 
+        # validate uniqueness of folder paths
+        if len(folder_map.values()) != len(set(folder_map.values())):
+            raise DuplicateFolderPaths(folders)
+
     def _get_default_folder(self, type):
         folder = default_settings["folder"][type]
         if folder is None:
@@ -1008,10 +1054,37 @@ class Settings(object):
 
         if migrate:
             self._migrate_config()
-        self._validate_config()
 
         self._forget_hashes()
 
+    def load_overlays(self, overlays, migrate=True):
+        for overlay in overlays:
+            if not os.path.exists(overlay):
+                continue
+
+            def process(path):
+                try:
+                    overlay_config = self.load_overlay(path, migrate=migrate)
+                    self.add_overlay(overlay_config)
+                    self._logger.info("Added config overlay from {}".format(path))
+                except Exception:
+                    self._logger.exception(
+                        "Could not add config overlay from {}".format(path)
+                    )
+
+            if os.path.isfile(overlay):
+                process(overlay)
+
+            elif os.path.isdir(overlay):
+                for entry in scandir(overlay):
+                    name = entry.name
+                    path = entry.path
+
+                    if is_hidden_path(path) or not fnmatch.fnmatch(name, "*.yaml"):
+                        continue
+
+                    process(path)
+
     def load_overlay(self, overlay, migrate=True):
         config = None
 
@@ -1578,15 +1651,6 @@ class Settings(object):
             return True
         return False
 
-    def _validate_config(self):
-        # validate uniqueness of folder paths
-        folder_keys = self.get(["folder"], merged=True).keys()
-        folders = dict(
-            (folder_key, self.getBaseFolder(folder_key)) for folder_key in folder_keys
-        )
-        if len(folders.values()) != len(set(folders.values())):
-            raise DuplicateFolderPaths(folders)
-
     def backup(self, suffix=None, path=None, ext=None, hidden=False):
         import shutil
 
@@ -1628,7 +1692,7 @@ class Settings(object):
                     self._config,
                     configFile,
                     default_flow_style=False,
-                    indent=4,
+                    indent=2,
                     allow_unicode=True,
                 )
                 self._dirty = False
@@ -1848,7 +1912,6 @@ class Settings(object):
         type,
         create=True,
         allow_fallback=True,
-        log_error=False,
         check_writable=True,
         deep_check_writable=False,
     ):
@@ -1869,23 +1932,20 @@ class Settings(object):
                 create=create,
                 check_writable=check_writable,
                 deep_check_writable=deep_check_writable,
-                log_error=log_error,
             )
         except Exception:
             if folder != default_folder and allow_fallback:
-                if log_error:
-                    self._logger.error(
-                        "Invalid configured {} folder at {}, attempting to "
-                        "fall back on default folder at {}".format(
-                            type, folder, default_folder
-                        )
+                self._logger.exception(
+                    "Invalid configured {} folder at {}, attempting to "
+                    "fall back on default folder at {}".format(
+                        type, folder, default_folder
                     )
+                )
                 _validate_folder(
                     default_folder,
                     create=create,
                     check_writable=check_writable,
                     deep_check_writable=deep_check_writable,
-                    log_error=log_error,
                 )
                 folder = default_folder
 
@@ -2163,9 +2223,7 @@ def _default_basedir(applicationName):
         return os.path.expanduser(os.path.join("~", "." + applicationName.lower()))
 
 
-def _validate_folder(
-    folder, create=True, check_writable=True, deep_check_writable=False, log_error=False
-):
+def _validate_folder(folder, create=True, check_writable=True, deep_check_writable=False):
     logger = logging.getLogger(__name__)
 
     if not os.path.exists(folder):
@@ -2178,8 +2236,7 @@ def _validate_folder(
             try:
                 os.makedirs(folder)
             except Exception:
-                if log_error:
-                    logger.exception("Could not create {}".format(folder))
+                logger.exception("Could not create {}".format(folder))
                 raise IOError(
                     "Folder for type {} at {} does not exist and creation failed".format(
                         type, folder
@@ -2211,6 +2268,5 @@ def _validate_folder(
                     f.write("test")
                 os.remove(testfile)
             except Exception:
-                if log_error:
-                    logger.exception("Could not write test file to {}".format(folder))
+                logger.exception("Could not write test file to {}".format(testfile))
                 raise IOError(error)
